@@ -1,9 +1,12 @@
+from decimal import Decimal
 from doctest import DocTestSuite
+from fractions import Fraction
 from functools import reduce
 from itertools import combinations, count, permutations
 from operator import mul
 from math import factorial
-from unittest import TestCase
+from sys import version_info
+from unittest import TestCase, skipIf
 
 import more_itertools as mi
 
@@ -100,7 +103,7 @@ class ConsumeTests(TestCase):
         self.assertEqual(0, next(r))
 
     def test_negative_consume(self):
-        """Check that negative consumsion throws an error"""
+        """Check that negative consumption throws an error"""
         r = (x for x in range(10))
         self.assertRaises(ValueError, lambda: mi.consume(r, -1))
 
@@ -200,7 +203,7 @@ class NcyclesTests(TestCase):
         n = mi.ncycles(range(100), 0)
         self.assertRaises(StopIteration, lambda: next(n))
 
-    def test_pathalogical_case(self):
+    def test_pathological_case(self):
         """asking for negative cycles should return an empty iterator"""
         n = mi.ncycles(range(100), -10)
         self.assertRaises(StopIteration, lambda: next(n))
@@ -824,7 +827,6 @@ class TriplewiseTests(TestCase):
 class SlidingWindowTests(TestCase):
     def test_basic(self):
         for iterable, n, expected in [
-            ([], 0, [()]),
             ([], 1, []),
             ([0], 1, [(0,)]),
             ([0, 1], 1, [(0,), (1,)]),
@@ -881,6 +883,26 @@ class PolynomialFromRootsTests(TestCase):
                 self.assertEqual(actual, expected)
 
 
+class PolynomialEvalTests(TestCase):
+    def test_basic(self):
+        for coefficients, x, expected in [
+            ([1, -4, -17, 60], 2, 18),
+            ([1, -4, -17, 60], 2.5, 8.125),
+            ([1, -4, -17, 60], Fraction(2, 3), Fraction(1274, 27)),
+            ([1, -4, -17, 60], Decimal('1.75'), Decimal('23.359375')),
+            ([], 2, 0),
+            ([], 2.5, 0.0),
+            ([], Fraction(2, 3), Fraction(0, 1)),
+            ([], Decimal('1.75'), Decimal('0.00')),
+            ([11], 7, 11),
+            ([11, 2], 7, 79),
+        ]:
+            with self.subTest(x=x):
+                actual = mi.polynomial_eval(coefficients, x)
+                self.assertEqual(actual, expected)
+                self.assertEqual(type(actual), type(x))
+
+
 class IterIndexTests(TestCase):
     def test_basic(self):
         iterable = 'AABCADEAF'
@@ -909,6 +931,11 @@ class IterIndexTests(TestCase):
                 expected = [0, 1, 4, 7]
                 self.assertEqual(actual, expected)
 
+    def test_stop(self):
+        actual = list(mi.iter_index('AABCADEAF', 'A', stop=7))
+        expected = [0, 1, 4]
+        self.assertEqual(actual, expected)
+
 
 class SieveTests(TestCase):
     def test_basic(self):
@@ -961,18 +988,26 @@ class BatchedTests(TestCase):
     def test_basic(self):
         iterable = range(1, 5 + 1)
         for n, expected in (
-            (0, []),
-            (1, [[1], [2], [3], [4], [5]]),
-            (2, [[1, 2], [3, 4], [5]]),
-            (3, [[1, 2, 3], [4, 5]]),
-            (4, [[1, 2, 3, 4], [5]]),
-            (5, [[1, 2, 3, 4, 5]]),
-            (6, [[1, 2, 3, 4, 5]]),
+            (1, [(1,), (2,), (3,), (4,), (5,)]),
+            (2, [(1, 2), (3, 4), (5,)]),
+            (3, [(1, 2, 3), (4, 5)]),
+            (4, [(1, 2, 3, 4), (5,)]),
+            (5, [(1, 2, 3, 4, 5)]),
+            (6, [(1, 2, 3, 4, 5)]),
         ):
             with self.subTest(n=n):
                 actual = list(mi.batched(iterable, n))
                 self.assertEqual(actual, expected)
 
+    def test_strict(self):
+        with self.assertRaises(ValueError):
+            list(mi.batched('ABCDEFG', 3, strict=True))
+
+        self.assertEqual(
+            list(mi.batched('ABCDEF', 3, strict=True)),
+            [('A', 'B', 'C'), ('D', 'E', 'F')],
+        )
+
 
 class TransposeTests(TestCase):
     def test_empty(self):
@@ -987,17 +1022,65 @@ class TransposeTests(TestCase):
         expected = [(10, 20, 30), (11, 21, 31), (12, 22, 32)]
         self.assertEqual(actual, expected)
 
-    def test_incompatible(self):
+    @skipIf(version_info[:2] < (3, 10), 'strict=True missing on 3.9')
+    def test_incompatible_error(self):
+        it = [(10, 11, 12, 13), (20, 21, 22), (30, 31, 32)]
+        with self.assertRaises(ValueError):
+            list(mi.transpose(it))
+
+    @skipIf(version_info[:2] >= (3, 9), 'strict=True missing on 3.9')
+    def test_incompatible_allow(self):
         it = [(10, 11, 12, 13), (20, 21, 22), (30, 31, 32)]
         actual = list(mi.transpose(it))
         expected = [(10, 20, 30), (11, 21, 31), (12, 22, 32)]
         self.assertEqual(actual, expected)
 
 
+class ReshapeTests(TestCase):
+    def test_empty(self):
+        actual = list(mi.reshape([], 3))
+        self.assertEqual(actual, [])
+
+    def test_zero(self):
+        matrix = [(0, 1, 2, 3), (4, 5, 6, 7), (8, 9, 10, 11)]
+        with self.assertRaises(ValueError):
+            list(mi.reshape(matrix, 0))
+
+    def test_basic(self):
+        matrix = [(0, 1, 2, 3), (4, 5, 6, 7), (8, 9, 10, 11)]
+        for cols, expected in (
+            (
+                1,
+                [
+                    (0,),
+                    (1,),
+                    (2,),
+                    (3,),
+                    (4,),
+                    (5,),
+                    (6,),
+                    (7,),
+                    (8,),
+                    (9,),
+                    (10,),
+                    (11,),
+                ],
+            ),
+            (2, [(0, 1), (2, 3), (4, 5), (6, 7), (8, 9), (10, 11)]),
+            (3, [(0, 1, 2), (3, 4, 5), (6, 7, 8), (9, 10, 11)]),
+            (4, [(0, 1, 2, 3), (4, 5, 6, 7), (8, 9, 10, 11)]),
+            (6, [(0, 1, 2, 3, 4, 5), (6, 7, 8, 9, 10, 11)]),
+            (12, [(0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11)]),
+        ):
+            with self.subTest(cols=cols):
+                actual = list(mi.reshape(matrix, cols))
+                self.assertEqual(actual, expected)
+
+
 class MatMulTests(TestCase):
     def test_n_by_n(self):
         actual = list(mi.matmul([(7, 5), (3, 5)], [[2, 5], [7, 9]]))
-        expected = [[49, 80], [41, 60]]
+        expected = [(49, 80), (41, 60)]
         self.assertEqual(actual, expected)
 
     def test_m_by_n(self):
@@ -1005,9 +1088,9 @@ class MatMulTests(TestCase):
         m2 = [[7, 11, 5, 4, 9], [3, 5, 2, 6, 3]]
         actual = list(mi.matmul(m1, m2))
         expected = [
-            [29, 47, 20, 38, 33],
-            [76, 122, 53, 82, 90],
-            [33, 53, 23, 36, 39],
+            (29, 47, 20, 38, 33),
+            (76, 122, 53, 82, 90),
+            (33, 53, 23, 36, 39),
         ]
         self.assertEqual(actual, expected)
 
@@ -1041,3 +1124,47 @@ class FactorTests(TestCase):
                 list(mi.factor(n)) == sorted(mi.factor(n)) for n in range(2000)
             )
         )
+
+
+class SumOfSquaresTests(TestCase):
+    def test_basic(self):
+        for it, expected in (
+            ([], 0),
+            ([1, 2, 3], 1 + 4 + 9),
+            ([2, 4, 6, 8], 4 + 16 + 36 + 64),
+        ):
+            with self.subTest(it=it):
+                actual = mi.sum_of_squares(it)
+                self.assertEqual(actual, expected)
+
+
+class PolynomialDerivativeTests(TestCase):
+    def test_basic(self):
+        for coefficients, expected in [
+            ([], []),
+            ([1], []),
+            ([1, 2], [1]),
+            ([1, 2, 3], [2, 2]),
+            ([1, 2, 3, 4], [3, 4, 3]),
+            ([1.1, 2, 3, 4], [(1.1 * 3), 4, 3]),
+        ]:
+            with self.subTest(coefficients=coefficients):
+                actual = mi.polynomial_derivative(coefficients)
+                self.assertEqual(actual, expected)
+
+
+class TotientTests(TestCase):
+    def test_basic(self):
+        for n, expected in (
+            (1, 1),
+            (2, 1),
+            (3, 2),
+            (4, 2),
+            (9, 6),
+            (12, 4),
+            (128_884_753_939, 128_884_753_938),
+            (999953 * 999983, 999952 * 999982),
+            (6**20, 1 * 2**19 * 2 * 3**19),
+        ):
+            with self.subTest(n=n):
+                self.assertEqual(mi.totient(n), expected)
